#python imports
from fastapi import APIRouter, HTTPException
from fastapi.responses import JSONResponse
from fastapi import FastAPI, Request
from fastapi.responses import HTMLResponse
from fastapi.templating import Jinja2Templates
from pydantic import BaseModel

#dependency imports
from entities.user import User
from use_cases.create_user import CreateUser, ICreateUserInputPort, ICreateUserOutputPort
from interface_adapters.sql_user_repository import SQLUserRepository


class UserDTO(BaseModel):
    id: int
    user_name: str
    user_email: str

class CreateUserRequest(BaseModel):
    user_name: str
    user_email: str
    user_password: str

class CreateUserResponse(BaseModel):
    status: str
    data: UserDTO

class CreateUserPresenter(ICreateUserOutputPort):
    def present(self, user:User) -> CreateUserResponse:
        """Format the response data for API."""
        if not user:
            return None
        user_dto = UserDTO(
            id = user.id, 
            user_name = user.name, 
            user_email = user.email)
        response = CreateUserResponse(status="success",data=user_dto)
        return response

class CreateUserController:
    def __init__(self, input_port: ICreateUserInputPort) -> None:
        self.input_port = input_port

    def handle_request(self, request:CreateUserRequest):
        """Handles the incoming API request and delegates to the use case."""
        return self.input_port.execute(request.user_name, request.user_email, request.user_password)
    
user_repository = SQLUserRepository()
create_user_presenter = CreateUserPresenter()
create_user_use_case = CreateUser(user_repository, create_user_presenter)
create_user_controller = CreateUserController(create_user_use_case)


router = APIRouter()
@router.post("/create_user", response_model=CreateUserResponse)
async def create_user(request:CreateUserRequest):
    response = create_user_controller.handle_request(request)
    if not response:
        raise HTTPException(status_code=500, detail="Could not create user")
    return response
